@@ -568,9 +568,11 @@ void _acb_poly_sqrt_series(acb_ptr g, acb_srcptr h, slong hlen, slong len, slong
 void acb_poly_sqrt_series(acb_poly_t g, const acb_poly_t h, slong n, slong prec);
 
 void _acb_poly_log_series(acb_ptr res, acb_srcptr f, slong flen, slong n, slong prec);
-
 void acb_poly_log_series(acb_poly_t res, const acb_poly_t f, slong n, slong prec);
 
+void _acb_poly_log1p_series(acb_ptr res, acb_srcptr f, slong flen, slong n, slong prec);
+void acb_poly_log1p_series(acb_poly_t res, const acb_poly_t f, slong n, slong prec);
+
 void _acb_poly_atan_series(acb_ptr res, acb_srcptr f, slong flen, slong n, slong prec);
 
 void acb_poly_atan_series(acb_poly_t res, const acb_poly_t f, slong n, slong prec);
@@ -580,6 +582,9 @@ void acb_poly_exp_series_basecase(acb_poly_t f, const acb_poly_t h, slong n, slo
 void _acb_poly_exp_series(acb_ptr f, acb_srcptr h, slong hlen, slong n, slong prec);
 void acb_poly_exp_series(acb_poly_t f, const acb_poly_t h, slong n, slong prec);
 
+void _acb_poly_exp_pi_i_series(acb_ptr f, acb_srcptr h, slong hlen, slong n, slong prec);
+void acb_poly_exp_pi_i_series(acb_poly_t f, const acb_poly_t h, slong n, slong prec);
+
 void _acb_poly_sinh_cosh_series_basecase(acb_ptr s, acb_ptr c, const acb_srcptr h, slong hlen, slong n, slong prec);
 void acb_poly_sinh_cosh_series_basecase(acb_poly_t s, acb_poly_t c, const acb_poly_t h, slong n, slong prec);
 void _acb_poly_sinh_cosh_series_exponential(acb_ptr s, acb_ptr c, const acb_srcptr h, slong hlen, slong n, slong prec);
@@ -645,6 +650,9 @@ void acb_poly_tan_series(acb_poly_t g, const acb_poly_t h, slong n, slong prec);
 void _acb_poly_sinc_series(acb_ptr g, acb_srcptr h, slong hlen, slong n, slong prec);
 void acb_poly_sinc_series(acb_poly_t g, const acb_poly_t h, slong n, slong prec);
 
+void _acb_poly_lambertw_series(acb_ptr res, acb_srcptr z, slong zlen, const fmpz_t k, int flags, slong len, slong prec);
+void acb_poly_lambertw_series(acb_poly_t res, const acb_poly_t z, const fmpz_t k, int flags, slong len, slong prec);
+
 void _acb_poly_gamma_series(acb_ptr res, acb_srcptr h, slong hlen, slong len, slong prec);
 
 void acb_poly_gamma_series(acb_poly_t res, const acb_poly_t f, slong n, slong prec);
