@@ -26,6 +26,14 @@
 extern "C" {
 #endif
 
+#define __ARB_VERSION 2
+#define __ARB_VERSION_MINOR 18
+#define __ARB_VERSION_PATCHLEVEL 1
+#define ARB_VERSION "2.18.1"
+#define __ARB_RELEASE (__ARB_VERSION * 10000 + \
+                         __ARB_VERSION_MINOR * 100 + \
+                         __ARB_VERSION_PATCHLEVEL)
+
 ARB_DLL extern const char * arb_version;
 double arb_test_multiplier(void);
 
@@ -159,8 +167,9 @@ void arb_neg_round(arb_t x, const arb_t y, slong prec);
 void arb_abs(arb_t y, const arb_t x);
 
 void arb_sgn(arb_t res, const arb_t x);
+int arb_sgn_nonzero(const arb_t x);
 
-void _arb_digits_round_inplace(char * s, mp_bitcnt_t * shift, fmpz_t error, slong n, arf_rnd_t rnd);
+void _arb_digits_round_inplace(char * s, flint_bitcnt_t * shift, fmpz_t error, slong n, arf_rnd_t rnd);
 
 int arb_set_str(arb_t res, const char * inp, slong prec);
 
@@ -291,57 +300,10 @@ arb_get_rad_arb(arb_t z, const arb_t x)
     mag_zero(arb_radref(z));
 }
 
-ARB_INLINE void
-arb_get_abs_ubound_arf(arf_t u, const arb_t x, slong prec)
-{
-    arf_t t;
-    arf_init_set_mag_shallow(t, arb_radref(x));
-
-    if (arf_sgn(arb_midref(x)) < 0)
-        arf_sub(u, arb_midref(x), t, prec, ARF_RND_UP);
-    else
-        arf_add(u, arb_midref(x), t, prec, ARF_RND_UP);
-
-    arf_abs(u, u);
-}
-
-ARB_INLINE void
-arb_get_abs_lbound_arf(arf_t u, const arb_t x, slong prec)
-{
-    arf_t t;
-    arf_init_set_mag_shallow(t, arb_radref(x));
-
-    if (arf_sgn(arb_midref(x)) > 0)
-    {
-        arf_sub(u, arb_midref(x), t, prec, ARF_RND_DOWN);
-    }
-    else
-    {
-        arf_add(u, arb_midref(x), t, prec, ARF_RND_DOWN);
-        arf_neg(u, u);
-    }
-
-    if (arf_sgn(u) < 0)
-        arf_zero(u);
-}
-
-ARB_INLINE void
-arb_get_ubound_arf(arf_t u, const arb_t x, long prec)
-{
-    arf_t t;
-    arf_init_set_mag_shallow(t, arb_radref(x));
-
-    arf_add(u, arb_midref(x), t, prec, ARF_RND_CEIL);
-}
-
-ARB_INLINE void
-arb_get_lbound_arf(arf_t u, const arb_t x, long prec)
-{
-    arf_t t;
-    arf_init_set_mag_shallow(t, arb_radref(x));
-
-    arf_sub(u, arb_midref(x), t, prec, ARF_RND_FLOOR);
-}
+void arb_get_abs_ubound_arf(arf_t u, const arb_t x, slong prec);
+void arb_get_abs_lbound_arf(arf_t u, const arb_t x, slong prec);
+void arb_get_ubound_arf(arf_t u, const arb_t x, slong prec);
+void arb_get_lbound_arf(arf_t u, const arb_t x, slong prec);
 
 void arb_nonnegative_part(arb_t res, const arb_t x);
 
@@ -353,6 +315,8 @@ arb_rel_accuracy_bits(const arb_t x)
     return -arb_rel_error_bits(x);
 }
 
+slong arb_rel_one_accuracy_bits(const arb_t x);
+
 ARB_INLINE slong
 arb_bits(const arb_t x)
 {
@@ -397,6 +361,8 @@ int arb_overlaps(const arb_t x, const arb_t y);
 
 int arb_contains(const arb_t x, const arb_t y);
 
+int arb_contains_interior(const arb_t x, const arb_t y);
+
 int arb_contains_int(const arb_t x);
 
 void arb_get_interval_fmpz_2exp(fmpz_t a, fmpz_t b, fmpz_t exp, const arb_t x);
@@ -457,6 +423,16 @@ void arb_submul_si(arb_t z, const arb_t x, slong y, slong prec);
 void arb_submul_ui(arb_t z, const arb_t x, ulong y, slong prec);
 void arb_submul_fmpz(arb_t z, const arb_t x, const fmpz_t y, slong prec);
 
+void arb_dot_simple(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, arb_srcptr y, slong ystep, slong len, slong prec);
+void arb_dot_precise(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, arb_srcptr y, slong ystep, slong len, slong prec);
+void arb_dot(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, arb_srcptr y, slong ystep, slong len, slong prec);
+
+void arb_approx_dot(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, arb_srcptr y, slong ystep, slong len, slong prec);
+
 void arb_div(arb_t z, const arb_t x, const arb_t y, slong prec);
 void arb_div_arf(arb_t z, const arb_t x, const arf_t y, slong prec);
 void arb_div_si(arb_t z, const arb_t x, slong y, slong prec);
@@ -790,36 +766,7 @@ _arb_vec_scalar_addmul(arb_ptr res, arb_srcptr vec,
         arb_addmul(res + i, vec + i, c, prec);
 }
 
-ARB_INLINE void
-_arb_vec_dot(arb_t res, arb_srcptr vec1, arb_srcptr vec2, slong len2, slong prec)
-{
-    slong i;
-    arb_zero(res);
-    for (i = 0; i < len2; i++)
-        arb_addmul(res, vec1 + i, vec2 + i, prec);
-}
-
-ARB_INLINE void
-_arb_vec_get_mag(mag_t bound, arb_srcptr vec, slong len)
-{
-    if (len < 1)
-    {
-        mag_zero(bound);
-    }
-    else
-    {
-        mag_t t;
-        slong i;
-        arb_get_mag(bound, vec);
-        mag_init(t);
-        for (i = 1; i < len; i++)
-        {
-            arb_get_mag(t, vec + i);
-            mag_max(bound, bound, t);
-        }
-        mag_clear(t);
-    }
-}
+void _arb_vec_get_mag(mag_t bound, arb_srcptr vec, slong len);
 
 ARB_INLINE slong
 _arb_vec_bits(arb_srcptr x, slong len)
@@ -836,23 +783,7 @@ _arb_vec_bits(arb_srcptr x, slong len)
     return b;
 }
 
-ARB_INLINE void
-_arb_vec_set_powers(arb_ptr xs, const arb_t x, slong len, slong prec)
-{
-    slong i;
-
-    for (i = 0; i < len; i++)
-    {
-        if (i == 0)
-            arb_one(xs + i);
-        else if (i == 1)
-            arb_set_round(xs + i, x, prec);
-        else if (i % 2 == 0)
-            arb_mul(xs + i, xs + i / 2, xs + i / 2, prec);
-        else
-            arb_mul(xs + i, xs + i - 1, x, prec);
-    }
-}
+void _arb_vec_set_powers(arb_ptr xs, const arb_t x, slong len, slong prec);
 
 ARB_INLINE void
 _arb_vec_add_error_arf_vec(arb_ptr res, arf_srcptr err, slong len)
@@ -971,11 +902,11 @@ int _arb_get_mpn_fixed_mod_log2(mp_ptr w, fmpz_t q, mp_limb_t * error,
 
 slong _arb_exp_taylor_bound(slong mag, slong prec);
 
-void _arb_exp_sum_bs_powtab(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
-    const fmpz_t x, mp_bitcnt_t r, slong N);
+void _arb_exp_sum_bs_powtab(fmpz_t T, fmpz_t Q, flint_bitcnt_t * Qexp,
+    const fmpz_t x, flint_bitcnt_t r, slong N);
 
-void _arb_exp_sum_bs_simple(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
-    const fmpz_t x, mp_bitcnt_t r, slong N);
+void _arb_exp_sum_bs_simple(fmpz_t T, fmpz_t Q, flint_bitcnt_t * Qexp,
+    const fmpz_t x, flint_bitcnt_t r, slong N);
 
 /* sin/cos implementation */
 
@@ -1020,12 +951,12 @@ void arb_sin_cos_wide(arb_t s, arb_t c, const arb_t x, slong prec);
 void _arb_sin_cos_generic(arb_t s, arb_t c, const arf_t x, const mag_t xrad, slong prec);
 void arb_sin_cos_generic(arb_t s, arb_t c, const arb_t x, slong prec);
 
-ARB_INLINE mp_bitcnt_t
+ARB_INLINE flint_bitcnt_t
 _arb_mpn_leading_zeros(mp_srcptr d, mp_size_t n)
 {
     mp_limb_t t;
     mp_size_t zero_limbs;
-    mp_bitcnt_t bits;
+    flint_bitcnt_t bits;
 
     zero_limbs = 0;
 
@@ -1046,11 +977,11 @@ _arb_mpn_leading_zeros(mp_srcptr d, mp_size_t n)
     }
 }
 
-void _arb_atan_sum_bs_simple(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
-    const fmpz_t x, mp_bitcnt_t r, slong N);
+void _arb_atan_sum_bs_simple(fmpz_t T, fmpz_t Q, flint_bitcnt_t * Qexp,
+    const fmpz_t x, flint_bitcnt_t r, slong N);
 
-void _arb_atan_sum_bs_powtab(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
-    const fmpz_t x, mp_bitcnt_t r, slong N);
+void _arb_atan_sum_bs_powtab(fmpz_t T, fmpz_t Q, flint_bitcnt_t * Qexp,
+    const fmpz_t x, flint_bitcnt_t r, slong N);
 
 void arb_atan_arf_bb(arb_t z, const arf_t x, slong prec);
 
@@ -1086,6 +1017,14 @@ _arb_vec_estimate_allocated_bytes(slong len, slong prec)
     return size;
 }
 
+int arb_load_str(arb_t res, const char * data);
+
+char * arb_dump_str(const arb_t x);
+
+int arb_load_file(arb_t res, FILE *stream);
+
+int arb_dump_file(FILE* stream, const arb_t x);
+
 #ifdef __cplusplus
 }
 #endif
